#ifndef _SRPCHOWLING_DETECT_H_
#define _SRPCHOWLING_DETECT_H_

#include "audiodsp/audio_dsp_impl.h"

#define HowDetVERSION float_c_v2_1_pc

#define WL 512
#define bandCnt 12
#define kFeatureNum 3
#define MAXsmoothW 2000
#define MAXRateW 100


class SRPCHowlingDetect : public IHowlingDetect
{
public:
	SRPCHowlingDetect();
	~SRPCHowlingDetect();

    //must bt 16khz sample rate and 1 channel
    virtual int Process(short* in, int Len, int sampleRate, int channel) override;
    virtual bool IsHowling() override;
    virtual void Enable(bool enable) override;
    
private:

    void SetParameter(int w, int k, int r);
    void Reset();

    //void circle_in(short* in, int Len);
    float SumAfun(float * A_buffer_, int bin0, int bin1);
    float MaxAfun(float * A_buffer_, int bin0, int bin1);
    // Input buffer where the original samples are stored.
    short circle_buffer_[WL];
    int circle_head;
    const float* window_;
    //int frameL;
    //for fft
    float fft_buffer_[WL];
    size_t ip_[WL];
    float wfft_[WL/2];

    float Apre_buffer_[WL/2];
    float A_buffer_[WL/2];
    float sumA_;
    //float feature_[bandCnt][kFeatureNum];
    float score_[bandCnt];
    float scores_[MAXsmoothW];
    float rates_[MAXsmoothW];
    int scoreshead;
    // calc rate
    float scoresforrate[MAXRateW];
    int scoresforratehead;
    //1=howlingdetect able, 0=howlingdetect disable
    int is_howlingdetect_able;
    int wSmooth;
    float kThresh;
    float rRate;
};

#endif

/*
-------- Real DFT / Inverse of Real DFT --------
[definition]
<case1> RDFT
R[k] = sum_j=0^n-1 a[j]*cos(2*pi*j*k/n), 0<=k<=n/2
I[k] = sum_j=0^n-1 a[j]*sin(2*pi*j*k/n), 0<k<n/2
<case2> IRDFT (excluding scale)
a[k] = (R[0] + R[n/2]*cos(pi*k))/2 +
sum_j=1^n/2-1 R[j]*cos(2*pi*j*k/n) +
sum_j=1^n/2-1 I[j]*sin(2*pi*j*k/n), 0<=k<n
[usage]
<case1>
ip[0] = 0; // first time only
rdft(n, 1, a, ip, w);
<case2>
ip[0] = 0; // first time only
rdft(n, -1, a, ip, w);
[parameters]
n              :data length (size_t)
n >= 2, n = power of 2
a[0...n-1]     :input/output data (float *)
<case1>
output data
a[2*k] = R[k], 0<=k<n/2
a[2*k+1] = I[k], 0<k<n/2
a[1] = R[n/2]
<case2>
input data
a[2*j] = R[j], 0<=j<n/2
a[2*j+1] = I[j], 0<j<n/2
a[1] = R[n/2]
ip[0...*]      :work area for bit reversal (size_t *)
length of ip >= 2+sqrt(n/2)
strictly,
length of ip >=
2+(1<<(int)(log(n/2+0.5)/log(2))/2).
ip[0],ip[1] are pointers of the cos/sin table.
w[0...n/2-1]   :cos/sin table (float *)
w[],ip[] are initialized if ip[0] == 0.
[remark]
Inverse of
rdft(n, 1, a, ip, w);
is
rdft(n, -1, a, ip, w);
for (j = 0; j <= n - 1; j++) {
a[j] *= 2.0 / n;
}
.
*/
